/*
  Copyright (c) 2013-2014, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#pragma once

#include "event.hpp"
#include "monitor.hpp"
#include "queue.hpp"

#include <cstdarg>
#include <list>
#include <memory>


namespace MMSS {

class context_t : public now_t, public nocopy_t {
private:
	timeout_queue_t<event_t> event_queue;
	std::list<std::shared_ptr<monitor_t>> monitors;

public:
	context_t() : event_queue(this) {}

	void queue_event(std::shared_ptr<event_t> &&data, uint64_t timeout) {
		event_queue.put(std::move(data), timeout);
	}

	void attach_monitor(std::shared_ptr<monitor_t> &&monitor) {
		monitors.emplace_back(std::move(monitor));
	}

	void node_log(node_t *node, int priority, const std::string &message) {
		for (auto monitor : monitors)
			monitor->handle_log(node, priority, message);
	}

	void node_init(node_t *node, const uint8_t *node_id, size_t len) {
		for (auto monitor : monitors)
			monitor->handle_init(node, node_id, len);
	}

	void node_neigh(node_t *node, gmrf_iface_t *iface, const gmrf_addr_t *addr, float rxcost, float txcost) {
		for (auto monitor : monitors)
			monitor->handle_neigh(node, iface, addr, rxcost, txcost);
	}

	void node_neigh_lost(node_t *node, gmrf_iface_t *iface, const gmrf_addr_t *addr) {
		for (auto monitor : monitors)
			monitor->handle_neigh_lost(node, iface, addr);
	}

	void node_route(node_t *node, const uint8_t *node_id, size_t len, gmrf_iface_t *iface, const gmrf_addr_t *addr, int metric) {
		for (auto monitor : monitors)
			monitor->handle_route(node, node_id, len, iface, addr, metric);
	}

	void node_route_lost(node_t *node, const uint8_t *node_id, size_t len) {
		for (auto monitor : monitors)
			monitor->handle_route_lost(node, node_id, len);
	}

	void logf(int priority, const char *format, ...);

	void run(int argc, char *argv[]);
};

}
